/**
 * Prosys OPC UA Java SDK
 *
 * Copyright (c) 2009-2010 Prosys PMS Ltd., <http://www.prosysopc.com>. 
 * All rights reserved.
 */
package com.prosysopc.ua.tests;

import java.io.IOException;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.util.Locale;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.opcfoundation.ua.builtintypes.DataValue;
import org.opcfoundation.ua.builtintypes.NodeId;
import org.opcfoundation.ua.builtintypes.Variant;
import org.opcfoundation.ua.core.EncodeableSerializer;
import org.opcfoundation.ua.core.Identifiers;
import org.opcfoundation.ua.encoding.DecodingException;
import org.opcfoundation.ua.encoding.EncodingException;
import org.opcfoundation.ua.encoding.IDecoder;
import org.opcfoundation.ua.encoding.IEncodeable;
import org.opcfoundation.ua.encoding.IEncoder;
import org.opcfoundation.ua.encoding.utils.AbstractSerializer;

import com.prosysopc.ua.SecureIdentityException;
import com.prosysopc.ua.StatusException;
import com.prosysopc.ua.nodes.UaObject;
import com.prosysopc.ua.samples.SampleConsoleServer;
import com.prosysopc.ua.server.NodeManagerUaNode;
import com.prosysopc.ua.server.UaServerException;
import com.prosysopc.ua.server.nodes.CacheVariable;
import com.prosysopc.ua.server.nodes.UaVariableNode;
import com.prosysopc.ua.server.nodes.opcua.FolderType;

/**
 * A sample OPC UA server application.
 */
public class ExtensionObjectTestServer extends SampleConsoleServer {

	private static Logger logger = Logger
			.getLogger(ExtensionObjectTestServer.class);

	/**
	 * @param args
	 *            command line arguments for the application
	 * @throws IOException
	 * @throws SecureIdentityException
	 * @throws StatusException
	 *             if the server address space creation fails
	 * @throws CertificateExpiredException
	 *             if the application certificate has expired (should not
	 *             happen, since we enable renewing it)
	 * @throws CertificateNotYetValidException
	 *             if the application certificate is not yet valid (should not
	 *             happen, since we are creating it ourselves)
	 * @throws UaServerException
	 *             if the server initialization parameters are invalid
	 */
	public static void main(String[] args) throws SecureIdentityException,
			IOException, UaServerException, StatusException {
		PropertyConfigurator.configure(SampleConsoleServer.class
				.getResource("log.properties"));

		ExtensionObjectTestServer server = new ExtensionObjectTestServer();
		server.initialize();
		server.run(getUseDiags(args));
	}

	private UaVariableNode eotsNode;
	private int extensionNamespaceIndex;
	private NodeManagerUaNode extensionNodeManager;
	private FolderType extensionVariableFolder;

	/**
	 * Create a sample address space with a new folder, a device object, a level
	 * variable, and an alarm condition.
	 * <p>
	 * The method demonstrates the basic means to create the nodes and
	 * references into the address space.
	 * <p>
	 * Simulation of the level measurement is defined in
	 * {@link #startSimulation()}
	 * 
	 * @throws StatusException
	 *             if the referred type nodes are not found from the address
	 *             space
	 * 
	 */
	@Override
	protected void createAddressSpace() throws StatusException {
		super.createAddressSpace();
		// My Node Manager
		extensionNodeManager = new NodeManagerUaNode(server,
				"http://www.prosysopc.com/OPCUA/ComplianceNodes");

		extensionNamespaceIndex = extensionNodeManager.getNamespaceIndex();

		// UA types and folders which we will use
		final UaObject objectsFolder = server.getNodeManagerRoot()
				.getObjectsFolder();

		// Folder for static test variables
		final NodeId staticVariableFolderId = new NodeId(
				extensionNamespaceIndex, "StaticVariablesFolder");
		extensionVariableFolder = new FolderType(extensionNodeManager,
				staticVariableFolderId, "StaticVariables", Locale.ENGLISH);
		extensionNodeManager.addNodeAndReference(objectsFolder,
				extensionVariableFolder, Identifiers.Organizes);

		final NodeId id = new NodeId(extensionNamespaceIndex, "EOTS");
		// UaType doubleType = server.getNodeManagerRoot().getType(
		// Identifiers.Double);
		eotsNode = new CacheVariable(extensionNodeManager, id, "EOTS",
				Locale.ENGLISH);
		// myLevel.setDataType(doubleType);
		// eotsNode.setTypeDefinition(myLevelType);
		ExtensionObjectTestStructure testStructure = new ExtensionObjectTestStructure();
		testStructure.setContent("TestString!!!");
		eotsNode.setValue(new DataValue(new Variant(testStructure)));
		objectsFolder.addReference(eotsNode, Identifiers.HasComponent, false);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.prosysopc.ua.samples.SampleConsoleServer#init()
	 */
	@Override
	protected void initialize() throws SecureIdentityException, IOException,
			UaServerException {
		super.initialize();

		// Add encoder for ExtensionObjectTestStructure
		EncodeableSerializer.getInstance().addSerializer(
				new AbstractSerializer(ExtensionObjectTestStructure.class,
						ExtensionObjectTestStructure.BINARY,
						ExtensionObjectTestStructure.XML) {
					@Override
					public void calcEncodeable(IEncodeable encodeable,
							IEncoder calculator) throws EncodingException {
						ExtensionObjectTestStructure obj = (ExtensionObjectTestStructure) encodeable;
						calculator.putString(null,
								(obj == null) ? null : obj.getContent());
					}

					@Override
					public IEncodeable getEncodeable(IDecoder decoder)
							throws DecodingException {
						ExtensionObjectTestStructure result = new ExtensionObjectTestStructure();
						result.setContent(decoder.getString("Content"));
						return result;
					}

					@Override
					public void putEncodeable(IEncodeable encodeable,
							IEncoder encoder) throws EncodingException {
						ExtensionObjectTestStructure obj = (ExtensionObjectTestStructure) encodeable;
						encoder.putString("Content",
								(obj == null) ? null : obj.getContent());
					}
				});
	}
}
